@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright (c) 2010, 2011, 2012.
+# Copyright (c) 2010, 2011, 2012, 2014.
 
 # SMHI,
 # Folkborgsvägen 1,
@@ -67,7 +67,10 @@ PROJNAME = {"vertical_perspective": "nsper",
             "geostationary": "geos",
             "albers_conical_equal_area": "aea",
             "azimuthal_equidistant": "aeqd",
-            
+            "equirectangular": "eqc",
+            "transverse_mercator": "tmerc",
+            "stereographic": "stere",
+            "general_oblique_transformation": "ob_tran"
     }
 
 def _load02(filename):
@@ -80,7 +83,8 @@ def _load02(filename):
     # processed variables
     processed = set()
 
-    satellite_name, satellite_number = rootgrp.platform.rsplit("-", 1)
+    # Currently MPOP does not like unicode (so much).
+    satellite_name, satellite_number = [str(i) for i in rootgrp.platform.rsplit("-", 1)]
 
     time_slot = rootgrp.variables["time"].getValue()[0]
     time_slot = num2date(time_slot, TIME_UNITS)
@@ -211,7 +215,7 @@ def _load02(filename):
                          units.lower().endswith("north") or 
                          units.lower().endswith("south")):
                         lats = coord_var[:]
-                if lons and lats:
+                if lons.any() and lats.any():
                     try:
                         from pyresample.geometry import SwathDefinition
                         area = SwathDefinition(lons=lons, lats=lats)
@@ -231,8 +235,9 @@ def _load02(filename):
             if len(names) == 1:
                 scales = np.array([scales])
                 offsets = np.array([offsets])
-            print scales, offsets
+            LOG.info("Scales and offsets: %s %s %s" % (str(names), str(scales), str(offsets)))
             for nbr, name in enumerate(names):
+                name = str(name)
                 try:
                     if cnt == 0:
                         chn_data = data[nbr, :, :].squeeze()
@@ -272,10 +277,12 @@ def _load02(filename):
         
             names = rootgrp.variables[dim][:]
             for nbr, name in enumerate(names):
+                name = str(name)
                 scene[name].wavelength_range[1] = var[nbr]
             try:
                 bnds = rootgrp.variables[var.bounds][:]
                 for nbr, name in enumerate(names):
+                    name = str(name)
                     scene[name].wavelength_range[0] = bnds[nbr, 0]
                     scene[name].wavelength_range[2] = bnds[nbr, 1]
                 processed |= set([var.bounds])
@@ -431,6 +438,7 @@ def load_from_nc4(filename):
                 LOG.debug("No lon/lat found.")
             
             for i, name in enumerate(names):
+                name = str(name)
                 if var.dimensions[0].startswith("band"):
                     chn_data = data[i, :, :]
                 elif var.dimensions[1].startswith("band"):
