@@ -23,8 +23,4 @@
 """Version file.
 """
 
-__major__ = "1"
-__minor__ = "0"
-__patch__ = "0"
-
-__version__ = "v" + ".".join([__major__, __minor__, __patch__])
+__version__ = "v1.3.1"
